#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_portal.interfaces import IPortalContext, IPortalPage, IPortalTemplateConfiguration, \
    IPortalPortletsConfiguration, IPortletRenderer, IPortlet
from pyams_skin.layer import IPyAMSUserLayer
from pyams_utils.interfaces import VIEW_SYSTEM_PERMISSION
from pyams_workflow.interfaces import IWorkflowPublicationInfo

# import packages
from pyams_default_theme.page import BaseIndexPage
from pyams_pagelet.pagelet import pagelet_config
from pyams_template.template import template_config
from pyramid.decorator import reify
from pyramid.exceptions import NotFound


@pagelet_config(name='', context=IPortalContext, layer=IPyAMSUserLayer)
@template_config(template='templates/pagelet.pt', layer=IPyAMSUserLayer)
class PortalContextIndexPage(BaseIndexPage):
    """Portal context index page"""

    def update(self):
        wf_info = IWorkflowPublicationInfo(self.context, None)
        if (wf_info is not None) and not wf_info.is_visible(self.request):
            raise NotFound()
        super(PortalContextIndexPage, self).update()

    @reify
    def page(self):
        return IPortalPage(self.context)

    @reify
    def template_configuration(self):
        return IPortalTemplateConfiguration(self.page.template)

    @reify
    def portlet_configuration(self):
        return IPortalPortletsConfiguration(self.context)

    def get_portlet(self, name):
        return self.request.registry.getUtility(IPortlet, name=name)

    def render_portlet(self, portlet_id):
        settings = self.portlet_configuration.get_portlet_configuration(portlet_id).settings
        renderer = self.request.registry.queryMultiAdapter((self.context, self.request, self, settings),
                                                           IPortletRenderer,
                                                           name=settings.renderer)
        if renderer is not None:
            return renderer()
        else:
            return ''


@pagelet_config(name='preview.html', context=IPortalContext, layer=IPyAMSUserLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class PortalContextPreviewPage(PortalContextIndexPage):
    """Portal context preview page"""

    def update(self):
        # Bypass publication status in preview
        super(PortalContextIndexPage, self).update()
