#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_form.interfaces.form import IInnerTabForm
from pyams_portal.interfaces import IPortlet, IPortalTemplate, IPortalPage, MANAGE_TEMPLATE_PERMISSION
from pyams_skin.layer import IPyAMSLayer

# import packages
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.url import absolute_url
from pyams_zmi.form import AdminDialogEditForm, InnerAdminEditForm
from pyramid.decorator import reify
from z3c.form import field
from zope.interface import Interface

from pyams_portal import _


@template_config(template='templates/portlet.pt', layer=IPyAMSLayer)
class PortletSettingsEditor(AdminDialogEditForm):
    """Portlet settings edit form"""

    prefix = 'portlet.'

    @property
    def title(self):
        translate = self.request.localizer.translate
        parent = self.configuration.__parent__
        if not IPortalTemplate.providedBy(parent):
            parent = IPortalPage(parent).template
        return translate(_("« {0} »  portal template - {1}")).format(parent.name,
                                                                     translate(self.portlet.label))

    legend = _("Edit portlet settings")
    dialog_class = 'modal-large'

    settings = None
    fields = field.Fields(Interface)
    edit_permission = MANAGE_TEMPLATE_PERMISSION

    @reify
    def configuration(self):
        return self.context.configuration

    @property
    def override_label(self):
        translate = self.request.localizer.translate
        parent = self.configuration.__parent__
        if IPortalTemplate.providedBy(parent):
            return translate(_("Override template settings"))
        else:
            page = IPortalPage(parent)
            if page.inherit_parent:
                return translate(_("Override parent settings"))
            else:
                return translate(_("Override template settings"))

    def get_form_action(self):
        return absolute_url(self.configuration.__parent__, self.request, self.request.view_name)

    def get_ajax_handler(self):
        return absolute_url(self.configuration.__parent__, self.request, self.ajax_handler)

    @reify
    def portlet(self):
        registry = self.request.registry
        return registry.queryUtility(IPortlet, name=self.configuration.portlet_name)


@adapter_config(name='properties', context=(Interface, IPyAMSLayer, PortletSettingsEditor), provides=IInnerTabForm)
class PortletSettingsPropertiesEditor(InnerAdminEditForm):
    """Portlet settings properties editor"""

    prefix = 'portlet_properties.'

    tab_label = _("Main properties")
    legend = None

    edit_permission = MANAGE_TEMPLATE_PERMISSION

    @property
    def fields(self):
        return field.Fields(self.parent_form.settings).omit('__name__')

    weight = 10
